/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: AdaptorActionBarContributor.java,v 1.7 2005/02/16 22:20:36 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.logging.adapter.ui.internal.presentation;


import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.edit.ui.action.CreateChildAction;
import org.eclipse.emf.edit.ui.action.EditingDomainActionBarContributor;
import org.eclipse.hyades.logging.adapter.model.internal.parser.ParserConfigType;
import org.eclipse.hyades.logging.adapter.model.internal.parser.RuleAttributeType;
import org.eclipse.hyades.logging.adapter.model.internal.parser.RuleElementType;
import org.eclipse.hyades.logging.adapter.ui.AcadEditorPlugin;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IContributionManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.SubContributionItem;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.ui.IEditorPart;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDSimpleTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;


/**
 * This is the action bar contributor for the Adaptor model editor.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class AdaptorActionBarContributor
	extends EditingDomainActionBarContributor
	implements ISelectionChangedListener {
	/**
	 * This keeps track of the active editor.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected IEditorPart activeEditorPart;

	/**
	 * This keeps track of the current selection provider.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ISelectionProvider selectionProvider;

	/**
	 * This will contain one {@link CreateChildAction} corresponding to each descriptor
	 * generated for the current selection by the item provider.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected Collection createChildActions;

	/**
	 * This is the menu manager into which menu contribution items should be added for CreateChild actions.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected IMenuManager createChildMenuManager;

	/**
	 * This creates an instance of the contributor.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public AdaptorActionBarContributor() {
	}

	/**
	 * This adds Separators for editor additions to the tool bar.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void contributeToToolBar(IToolBarManager toolBarManager) {
		toolBarManager.add(new Separator("adaptor-settings"));
		toolBarManager.add(new Separator("adaptor-additions"));
	}

	/**
	 * This adds to the menu bar a menu and some separators for editor additions,
	 * as well as the sub-menus for object creation items.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void contributeToMenu(IMenuManager menuManager) {
		super.contributeToMenu(menuManager);
	
		IMenuManager submenuManager = new MenuManager(AcadEditorPlugin.INSTANCE.getString("_UI_AdaptorEditor_menu"), "org.eclipse.hyades.acad.model.internal.adaptorMenuID");
		menuManager.insertAfter("additions", submenuManager);
		submenuManager.add(new Separator("settings"));
		submenuManager.add(new Separator("actions"));
		submenuManager.add(new Separator("additions"));
		submenuManager.add(new Separator("additions-end"));
	
		// Prepare for CreateChild item addition or removal.
		//
		createChildMenuManager = new MenuManager(AcadEditorPlugin.INSTANCE.getString("_UI_CreateChild_menu_item"));
		submenuManager.insertBefore("additions", createChildMenuManager);
	}

	/**
	 * When the active editor changes,
	 * this remembers the change,
	 * and registers with it as a selection provider.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setActiveEditor(IEditorPart part) {
		super.setActiveEditor(part);
		activeEditorPart = part;

		// Switch to the new selection provider.
		//
		if (selectionProvider != null) {
			selectionProvider.removeSelectionChangedListener(this);
		}
		if (part == null) {
			selectionProvider = null;
		}
		else {
			selectionProvider = part.getSite().getSelectionProvider();
			selectionProvider.addSelectionChangedListener(this);

			// Fake a selection changed event to update the menus.
			//
			if (selectionProvider.getSelection() != null) {
				selectionChanged(new SelectionChangedEvent(selectionProvider, selectionProvider.getSelection()));
			}
		}
	}

	/**
	 * This implements {@link ISelectionChangedListener},
	 * handling {@link SelectionChangedEvents} by querying for the children and siblings
	 * that can be added to the selected object and updating the menus accordingly.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		// Remove any menu items for old selection.
		//
		if (createChildMenuManager != null) {
			depopulateManager(createChildMenuManager, createChildActions);
		}

		// Query the new selection for appropriate new child/sibling descriptors
		//
		Collection newChildDescriptors = null;

		ISelection selection = event.getSelection();
		if (selection instanceof IStructuredSelection && ((IStructuredSelection)selection).size() == 1) {
			Object object = ((IStructuredSelection)selection).getFirstElement();

			EditingDomain domain =
				((IEditingDomainProvider) activeEditorPart).getEditingDomain();

			newChildDescriptors = domain.getNewChildDescriptors(object, null);
		}

		// Generate actions for selection; populate and redraw the menus.
		//
		createChildActions = generateCreateChildActions(newChildDescriptors, selection);

		if (createChildMenuManager != null) {
			populateManager(createChildMenuManager, createChildActions, null);
			createChildMenuManager.update(true);
		}
	}

	/**
	 * This generates a {@link CreateChildAction} for each object in <code>descriptors</code>,
	 * and returns the collection of these actions.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected Collection generateCreateChildActions(Collection descriptors, ISelection selection) {
		Collection actions = new LinkedList();
		Object object = ((IStructuredSelection)selection).getFirstElement();
		
		if (descriptors != null) {
			for (Iterator i = descriptors.iterator(); i.hasNext(); ) {
				
				CommandParameter command = (CommandParameter)i.next();
				CreateChildAction action = new CreateChildAction(activeEditorPart, selection, command);
				actions.add(action);
				
				if(object instanceof ParserConfigType)
				{ 
					Object value = command.getValue();
					action.setText(((RuleElementType)value).getName());
					
					EList rules = ((ParserConfigType)object).getRuleElement();
					for(int idx=0; idx<rules.size(); idx++)
					{
						RuleElementType elem = (RuleElementType) rules.get(idx);
						if(elem.getName() != null && elem.getName().equals(action.getText()))
						{
							action.setEnabled(false);
							break;
							
						}
					}
				}
				else if(object instanceof RuleElementType)
				{
					Object value = command.getValue();
					if(value != null && value instanceof RuleAttributeType)
					{
						action.setText(((RuleAttributeType)value).getName());
						enableAttributeAction(action, (RuleElementType)object);					
					}
					if(value != null && value instanceof RuleElementType)
					{
						action.setText(((RuleElementType)value).getName());	
						enableElementAction(action, (RuleElementType)object);											
					}					
				}				
			}
		}
		
		return actions;
	}

	/**
	 * This populates the specified <code>manager</code> with {@link ActionContributionItem}s
	 * based on the {@link IAction}s contained in the <code>actions</code> collection,
	 * by inserting them before the specified contribution item <code>contributionID</code>.
	 * If <code>ID</code> is <code>null</code>, they are simply added.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected void populateManager(IContributionManager manager, Collection actions, String contributionID) {
		if (actions != null) {
			for (Iterator i = actions.iterator(); i.hasNext(); ) {
				IAction action = (IAction) i.next();
				if (contributionID != null) {
					manager.insertBefore(contributionID, action);
				}
				else {
					manager.add(action);
				}
			}
		}
	}
		
	/**
	 * This removes from the specified <code>manager</code> all {@link ActionContributionItem}s
	 * based on the {@link IAction}s contained in the <code>actions</code> collection.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected void depopulateManager(IContributionManager manager, Collection actions) {
		if (actions != null) {
			IContributionItem[] items = manager.getItems();
			for (int i = 0; i < items.length; i++) {
				// Look into SubContributionItems
				//
				IContributionItem contributionItem = items[i];
				while (contributionItem instanceof SubContributionItem) {
					contributionItem = ((SubContributionItem)contributionItem).getInnerItem();
				}

				// Delete the ActionContributionItems with matching action.
				//
				if (contributionItem instanceof ActionContributionItem) {
					IAction action = ((ActionContributionItem) contributionItem).getAction();
					if (actions.contains(action)) {
						manager.remove(contributionItem);
					}
				}
			}
		}
	}

	/**
	 * This populates the pop-up menu before it appears.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void menuAboutToShow(IMenuManager menuManager) {
		super.menuAboutToShow(menuManager);
		MenuManager submenuManager = null;
	
		submenuManager = new MenuManager(AcadEditorPlugin.INSTANCE.getString("STR_ADD"));
		populateManager(submenuManager, createChildActions, null);
		menuManager.insertBefore("additions", submenuManager);
	}
	
	protected void enableAttributeAction(Action action, RuleElementType object)
	{
		String name = action.getText();
		if(name == null)
		  return;
		  
		Object cbeObject = AcadEditorPlugin.INSTANCE.getElementDefinitionForType(object);
		
		if(cbeObject == null)
		  return;
		  
		XSDTypeDefinition def = null;
		if(cbeObject instanceof XSDTypeDefinition)
		   def = (XSDTypeDefinition) cbeObject;
		else if(cbeObject instanceof XSDElementDeclaration){
		   def = ((XSDElementDeclaration)cbeObject).getTypeDefinition();
			Object decl = AcadEditorPlugin.INSTANCE.getElementDefinitionFromType(((XSDElementDeclaration)cbeObject).getType(), name);
			if(decl != null && decl instanceof XSDElementDeclaration) {
				enableSimpleTypeElementAction(action, (XSDElementDeclaration) decl, object);
				return;
			}
		}		
		
		if(def == null || def instanceof XSDSimpleTypeDefinition)
		{
			//this is an array attribute and multiple values are allowed
			return;
		}
		  
		EList attributes = object.getRuleAttribute();
		for(int idx=0; idx<attributes.size(); idx++)
		{
			if(name.equals(((RuleAttributeType)attributes.get(idx)).getName()))
			{
				action.setEnabled(false);
				return;      
			}
		}
		
		action.setEnabled(true);
	}
	
	protected void enableSimpleTypeElementAction(Action action, XSDElementDeclaration decl, RuleElementType object){
		String name = action.getText();
		if (name.equals("values")) {
			EList attributes = object.getRuleAttribute();
			for(int idx=0; idx<attributes.size(); idx++)
			{
				RuleAttributeType attr = (RuleAttributeType)attributes.get(idx);
				if(attr.getName().equals("hexValue"))
				{
					action.setEnabled(false);
					break;
				}
			}
			return;
		}
		if (name.equals("hexValue"))
		{
			EList attributes = object.getRuleAttribute();
			for(int idx=0; idx<attributes.size(); idx++)
			{
				RuleAttributeType attr = (RuleAttributeType)attributes.get(idx);
				if(attr.getName().equals("values") || attr.getName().equals("hexValue"))
				{
					action.setEnabled(false);
					break;
				}
			}
			return;
		}
		String value = decl.getElement().getAttribute("maxOccurs");
		if(value != null && value.equals("1"))
		{
			EList attributes = object.getRuleAttribute();
			for(int idx=0; idx<attributes.size(); idx++)
			{
				RuleAttributeType attr = (RuleAttributeType)attributes.get(idx);
				if(name.equals(attr.getName()))
				{
					action.setEnabled(false);
				}
			}
		}		
	}
	
	protected void enableElementAction(Action action, RuleElementType object)
	{
		String name = action.getText();
		
		Object cbeObject = AcadEditorPlugin.INSTANCE.getElementDefinitionForType(object);
		if(cbeObject != null && cbeObject instanceof XSDElementDeclaration)
		{			
			Object decl = AcadEditorPlugin.INSTANCE.getElementDefinitionFromType(((XSDElementDeclaration)cbeObject).getType(), name);
			if(decl == null || !(decl instanceof XSDElementDeclaration) )
			{
				EList elements = object.getRuleElement();
				for(int idx=0; idx<elements.size(); idx++)
				{
					RuleElementType relem = (RuleElementType)elements.get(idx);
					if(name.equals(relem.getName()))
					{
						action.setEnabled(false);
					}
				}
				
				return;
			}
			
			if (name.equals("values"))
			{
				EList elements = object.getRuleElement();
				for(int idx=0; idx<elements.size(); idx++)
				{
					RuleElementType relem = (RuleElementType)elements.get(idx);
					if(relem.getName().equals("hexValue"))
					{
						action.setEnabled(false);
						break;
					}
				}
				return;
			}
			if (name.equals("hexValue"))
			{
				EList elements = object.getRuleElement();
				for(int idx=0; idx<elements.size(); idx++)
				{
					RuleElementType relem = (RuleElementType)elements.get(idx);
					if(relem.getName().equals("values") || relem.getName().equals("hexValue"))
					{
						action.setEnabled(false);
						break;
					}
				}
				return;
			}
			String value = ((XSDElementDeclaration)decl).getElement().getAttribute("maxOccurs");
			if(value != null && value.equals("1"))
			{
				EList elements = object.getRuleElement();
				for(int idx=0; idx<elements.size(); idx++)
				{
					RuleElementType relem = (RuleElementType)elements.get(idx);
					if(name.equals(relem.getName()))
					{
						action.setEnabled(false);
					}
				}
			}			
		}
		else
		{
			EList elements = object.getRuleElement();
			for(int idx=0; idx<elements.size(); idx++)
			{
				RuleElementType relem = (RuleElementType)elements.get(idx);
				if(name.equals(relem.getName()))
				{
					action.setEnabled(false);
				}
			}

		}
	}
	
	
	protected void enableComplexType(Action action, XSDComplexTypeDefinition tdef, RuleElementType object)
	{
		String name = action.getText();
		String value = tdef.getElement().getAttribute("maxOccurs");
		if(value != null && value.equals("1"))
		{
			EList elements = object.getRuleElement();
			for(int i=0; i<elements.size(); i++)
			{
				RuleElementType relem = (RuleElementType)elements.get(i);
				if(name.equals(relem.getName()))
				{
					action.setEnabled(false);
					return;      
				}
			}
		}
		
		action.setEnabled(true);
	}
}
